#!/bin/sh

CFG_DIR_NAME=config
STAT_DIR_NAME=status
CNTRS_DIR_NAME=counters

DESC_FILE_NAME=readme

TMP_DIR=/tmp

SUFFIX_MIN=.min
SUFFIX_MAX=.max


# function progress
# This function prints out one symbol
# which if inserted one by one builds "progress bar"
progress ( )
{
  echo -n "."
}

# describe_error prints out string
# which describes error
#  $1 - is a string, describing error
describe_error ( )
{
  echo "* $1"
}

# report_failed prints out "failed" message
report_failed ( )
{
  echo "failed"
}

# report_ok prints out "ok" message
report_ok ( )
{
  echo "OK"
}

# check_directory_existance
# functions checks whether directory exists or not.
#   $1 - directory to check
check_directory_existance()
{
  test -d $1
  if [ $? -ne 0 ]; then
    echo "Directory <$1> does not exist"
    return 1
  fi
  return 0
}

# compare_files
# compare two files. 
# for example, one is result of measurements, second - is
# pattern results for first. comparing can be made with 3 rules: 
# 1 measurement result == pattern results - comparing files has same extension
# 2 measurement result < pattern results - pattern file has .max extension
# 3 measurement result > pattern results - pattern file has .min extension
# 
# parameters:
#   $1 measurement results file
#   $2 pattern file
compare_files ()
{
  echo -n "comparing <$1> and <$2>"
 
  local patt_file=$2
  local meas_file=$1
 
  # retrieving parameter list from patter file
  patt_params=`cat $patt_file | awk '{print $1}'`

  # for each parameter listed in patt_file
  for param in $patt_params
  do
    # reporting status    
    progress

    # get parameter's pattern and measured value 
    # FIXME: in the future there can be more then 1 columns
    # of values in files. 
    patt_value=`grep "\<$param\>" $patt_file | awk '{print $2}'`
    meas_value=`grep "\<$param\>" $meas_file | awk '{print $2}'`

    #echo "$patt_value <-> $meas_value"

    # checking that such parameter is present in meas file
    if ! grep $param $meas_file > /dev/null
    then
      report_failed
      describe_error "file <$meas_file> does not contain <$param> record"
      return 1
    fi

    # comparing values
    if test "`echo $patt_file | grep ".min" `" = $patt_file 
    then
      # pattern value must be less or equal to measured
      if [ $patt_value -gt $meas_value ]
      then
        report_failed
        describe_error "<$param> value is less"
        describe_error "than pattern. meas=$meas_value, patt=$patt_value"
        return 1
      fi
    elif test "`echo $patt_file | grep ".max" `" = $patt_file
    then
      # pattern value must be greater or equal to measured
      if [ $patt_value -lt $meas_value ]
      then
        report_failed
        describe_error "<$param> value is greater"
        describe_error "than pattern. meas=$meas_value, patt=$patt_value"
        return 1
      fi
    else
      # pattern value must be equal to measured
      if ! test $patt_value = $meas_value
      then
        report_failed
        describe_error "<$param> value is not equal"
        describe_error "to pattern. meas=$meas_value, patt=$patt_value"
        return 1
      fi
    fi
  done
 
  report_ok
 
  return 0 
}

# read_and_compare
# read results of measurements and compare it
# with reference values
# parameters
#  $1 /proc path to e1 card's measurements directory, 
#     for ex. - status/ or counters/
#  $2 path to directory which contains reference files to compare with
read_and_compare( )
{
  local meas_dir=$1
  local ref_dir=$2

  # 1. list ref_dir and check files existance.
  #    All files in ref_dir/ must be present in meas_dir,
  #    except .min & .max files.
  local f
  for f in `ls $ref_dir/ | grep -v "\.min" | grep -v "\.max"`
  do
    # check whether file $f exists in $meas_dir or not.
    if [ ! -e $meas_dir/$f ]
    then
      describe_error "cant compare <$ref_dir/$f> and <$meas_dir/$f>"
      describe_error "because <$meas_dir/$f> does not exist"
      return 1
    fi
  done

  # 2. list files in meas_dir and search reference file(s)
  #    for it. One file in meas_dir can have one or more than 
  #    one reference file or no reference files. 
  #    In case of multiple references,
  #    reference files can be named 1) similar to analogical file 
  #    in $ref_dir or/and 2) with ".min" suffix or/and 3) with ".max" suffix.

  # for each file in meas_dir
  for f in `ls $meas_dir/`
  do
    # if reference directory contains file $f
    # (possibly with different expression)
    for ref in `ls $ref_dir/ | grep $f`
    do
      # comparing files
      compare_files $meas_dir/$f $ref_dir/$ref || return 1
    done
  done
  
  return 0
}

VERSION="0.2.0"
get_txphase()
{
  local exp=$1
  local file="/sys/bus/exp$exp/devices/fpga-e1/info"
  local ver=`echo $VERSION | sed 's/\./ /g'`
  local cur_ver=`cat $file |grep version| awk '{print $2}'|sed 's/\./ /g'`

  for i in 1 2 3; do
    it_ver=`echo $ver | awk '{print $'$i'}'`
    it_cur_ver=`echo $cur_ver | awk '{ print $'$i'}'`
    if [ $it_ver -gt $it_cur_ver ]; then
       echo 0xA7C5AC
       return 0
    fi
    if [ $it_ver -lt $it_cur_ver ]; then
      echo 0xAAAAAA 
      return 0
    fi
    done
  echo 0xAAAAAA
  return 0
}

# apply_config
# Apply configuration from source config files to destination /proc files.
# In simple words, function performs `cat cfgfile > /proc/.../cfgfile`
# for each config file, located in source directory.
# After applying of configuration function compares source & destination
# config files. Function succeeds only if they are equal.
# NOTE: it is assumed, that source file's config records is present
#       in list with newline after each initialisation record. example:
#       a=b
#       c=d
#   $1 /proc/.../config/ directory for e1 card
#   $2 configuration source directory, example /home/../config/
apply_config( )
{
  local proc_cfg=$1
  local src_cfg=$2
  local expnum=$3
  local f

  echo -n "applying configuration <$src_cfg>"

  # for each file in src directory
  # NOTE: flag '-r' helps to apply 'command' configuration
  # after all other files
  for f in `ls -r $src_cfg`
  do
    progress
    # apply config
    cat $src_cfg/$f > $proc_cfg/$f
    if [ $? -ne 0 ]
    then
      report_failed
      describe_error "source file - <$src_cfg/$f>"
      describe_error "target file - <$proc_cfg/$f>"
      return 1
    fi
  done

  # getting txphase for this version of fw and write it to config
  local phase=`get_txphase $expnum`
  echo "txphase=$phase" > $proc_cfg/phys

  report_ok
  return 0
}

# check_config
# Check configuration equality between source config files 
# and destination /proc files.
# NOTE: it is assumed, that source file's config records is present
#       as list with newline after each initialization record. example:
#       a=b
#       c=d
#   $1 /proc/.../config/ just configured destination config directory 
#   $2 source configuration directory
check_config( )
{
  local proc_cfg=$1
  local src_cfg=$2
  local f

  echo "checking configuration"

  for f in `ls $src_cfg`
  do
    # comparing source & destination
    # but at first removing '=' characters
    # from source files
    cat $src_cfg/$f | sed "s/=/ /" > $TMP_DIR/$USER/$f
    compare_files $proc_cfg/$f $TMP_DIR/$USER/$f
    if [ $? -ne 0 ]
    then
      report_failed
      describe_error "source and destination config files are differs"
      return 1
    fi    
  done
  
  report_ok

  return 0
}


# run_test
# run one test
# parameters
#   $1 /proc path to e1 card's files
#   $2 initial configuration directory
#   $3 path to directory of test to run
run_test( )
{
  local proc_dir=$1
  local init_dir=$2
  local test_dir=$3

  # assume that temporary directory exists
  mkdir -p $TMP_DIR/$USER
    
  # it is assumed that /proc/.. and init directory are exists
  # 1. check directory existance
  # 1.a
  for d in $test_dir $init_dir
  do
    check_directory_existance $d || return 1
  done
  
  # 1.b let user know what this test does
  echo "-----------------------------"
  cat $test_dir/$DESC_FILE_NAME

  echo "preparing test $test_dir"

  # 2. apply initial configuration
  # Initial configuration can be applied not only 
  # to config/ files. So, scanning init/ directory.
  local f
  for f in `ls -l $init_dir | grep ^d | awk '{print $9}'`
  do
    if [ -e $proc_dir/$f ]
    then
      apply_config $proc_dir/$f $init_dir/$f || return 1
     
      # checking configuration for every config/ file
      if test $f = $CFG_DIR_NAME
      then
        check_config $proc_dir/$f $init_dir/$f || return 1
      fi
    else
      report_failed
      describe_error "file <$proc_dir/$f> does not exist"
    fi
  done
 
  # 3. apply test specific configuration if needed
  local test_cfg_dir=$test_dir/$CFG_DIR_NAME
  if [ -d $test_cfg_dir ]   
  then
    # applying test specific config if config 
    # directory is present for test
    apply_config $proc_dir/$CFG_DIR_NAME $test_cfg_dir || return 1
    check_config $proc_dir/$CFG_DIR_NAME $test_cfg_dir || return 1
  fi

  # 4. Wait for 2 seconds
  # FIXME: this value must be customizable!
  echo "test configured. wait 2 sec, please..."
  sleep 2

  # 5. read & compare results

  echo "comparing results" 
  
  # for each subdirectory in test directory
  for d in `ls -l $test_dir/| grep ^d| grep -v $CFG_DIR_NAME| awk '{print $9}'`
  do
    # except 'config' directory
    if ! test $d = $CFG_DIR_NAME
    then
      # read files in proc_dir and compare them with 
      # reference files in test_dir
      if [ -e $proc_dir/$d ] 
      then
        read_and_compare $proc_dir/$d $test_dir/$d || return 1
      else
        report_failed
        describe_error "cant compare <$test_dir/$d> and <$proc_dir/$d>"
        describe_error "because <$proc_dir/$d> does not exist."
        return 1
      fi
    fi
  done

  return 0
}

